@@ -1,5 +1,5 @@
 from .permission_map import PermissionMap
 
-default_app_config = 'bridgekeeper.apps.BridgekeeperConfig'
+default_app_config = "bridgekeeper.apps.BridgekeeperConfig"
 
 perms = PermissionMap()
