@@ -2,8 +2,9 @@ from django.apps import AppConfig
 
 
 class BridgekeeperConfig(AppConfig):
-    name = 'bridgekeeper'
+    name = "bridgekeeper"
 
     def ready(self):
         from django.utils.module_loading import autodiscover_modules
-        autodiscover_modules('permissions')
+
+        autodiscover_modules("permissions")
