@@ -4,7 +4,8 @@ This module defines the :class:`Rule` base class, as well as a
 number of built-in rules.
 """
 
-from django.db.models import Q, QuerySet
+from django.db.models import Manager, Q, QuerySet
+from django.db.models.fields.reverse_related import ForeignObjectRel
 
 
 class Sentinel:
@@ -170,8 +171,7 @@ class And(BinaryCompositeRule):
         return left & right
 
     def check(self, user, instance=None):
-        return (self.left.check(user, instance) and
-                self.right.check(user, instance))
+        return self.left.check(user, instance) and self.right.check(user, instance)
 
 
 class Or(BinaryCompositeRule):
@@ -200,8 +200,7 @@ class Or(BinaryCompositeRule):
         return left | right
 
     def check(self, user, instance=None):
-        return (self.left.check(user, instance) or
-                self.right.check(user, instance))
+        return self.left.check(user, instance) or self.right.check(user, instance)
 
 
 class Not(Rule):
@@ -285,9 +284,106 @@ def is_active(user):
     return user.is_active
 
 
+class R(Rule):
+    """Rules that allow access to some objects but not others.
+
+    ``R`` takes a set of field lookups as keyword arguments.
+
+    Each argument is a model attribute. Foreign keys can be traversed
+    using double underscores, as in :class:`~django.db.models.Q`
+    objects.
+
+    The value assigned to each argument can be:
+
+    - A value to match.
+    - A function that accepts a user, and returns a value to match.
+    - If the argument refers to a foreign key or many-to-many
+      relationship, another :class:`~bridgekeeper.rules.Rule` object.
+    """
+
+    def __init__(self, **kwargs):
+        self.kwargs = kwargs
+
+    def __repr__(self):
+        return "R({})".format(
+            ", ".join("{}={!r}".format(k, v) for k, v in self.kwargs.items())
+        )
+
+    def check(self, user, instance=None):
+        if instance is None:
+            return False
+
+        # This loop exits early, returning False, if any argument
+        # doesn't match.
+        for key, value in self.kwargs.items():
+
+            # Find the appropriate LHS on this object, traversing
+            # foreign keys if necessary.
+            lhs = instance
+            for key_fragment in key.split("__"):
+                field = lhs.__class__._meta.get_field(key_fragment,)
+                if isinstance(field, ForeignObjectRel):
+                    attr = field.get_accessor_name()
+                else:
+                    attr = key_fragment
+                lhs = getattr(lhs, attr)
+
+            # Compare it against the RHS.
+            # Note that the LHS will usually be a value, but in the case
+            # of a ManyToMany or the 'other side' of a ForeignKey it
+            # will be a RelatedManager. In this case, we need to check
+            # if there is at least one model that matches the RHS.
+            if isinstance(value, Rule):
+                if isinstance(lhs, Manager):
+                    if not value.filter(user, lhs.all()).exists():
+                        return False
+                else:
+                    if not value.check(user, lhs):
+                        return False
+            else:
+                resolved_value = value(user) if callable(value) else value
+                if isinstance(lhs, Manager):
+                    if resolved_value not in lhs.all():
+                        return False
+                else:
+                    if lhs != resolved_value:
+                        return False
+
+        # Woohoo, everything matches!
+        return True
+
+    def query(self, user):
+        accumulated_q = Q()
+
+        for key, value in self.kwargs.items():
+            # TODO: check lookups are not being used
+            if isinstance(value, Rule):
+                child_q_obj = value.query(user)
+                accumulated_q &= add_prefix(child_q_obj, key)
+            else:
+                resolved_value = value(user) if callable(value) else value
+                accumulated_q &= Q(**{key: resolved_value})
+
+        return accumulated_q
+
+
 class Attribute(Rule):
     """Rule class that checks the value of an instance attribute.
 
+    .. deprecated:: 0.9
+
+        Use :class:`~bridgekeeper.rules.R` objects instead.
+
+        ::
+
+            # old
+            Attribute('colour', matches='blue')
+            Attribute('tenant', lambda user: user.tenant)
+
+            # new
+            R(colour='blue')
+            R(tenant=lambda user: user.tenant)
+
     This rule is satisfied by model instances where the attribute
     given in ``attr`` matches the value given in ``matches``.
 
@@ -400,7 +496,7 @@ class In(Rule):
     def query(self, user):
         collection = self.get_collection(user)
         if isinstance(collection, QuerySet):
-            return Q(pk__in=collection.values_list('pk'))
+            return Q(pk__in=collection.values_list("pk"))
         return Q(pk__in=[x.pk for x in collection])
 
     def check(self, user, instance=None):
@@ -430,7 +526,7 @@ def add_prefix(q_obj, prefix):
         *(
             add_prefix(child, prefix)
             if isinstance(child, Q)
-            else (prefix + '__' + child[0], child[1])
+            else (prefix + "__" + child[0], child[1])
             for child in q_obj.children
         ),
         _connector=q_obj.connector,
@@ -441,6 +537,18 @@ def add_prefix(q_obj, prefix):
 class Relation(Rule):
     """Check that a rule applies to a ForeignKey.
 
+    .. deprecated:: 0.9
+
+        Use :class:`~bridgekeeper.rules.R` objects instead.
+
+        ::
+
+            # old
+            Relation('applicant', perms['foo.view_applicant'])
+
+            # new
+            R(applicant=perms['foo.view_applicant'])
+
     :param attr: Name of a foreign key attribute to check.
     :type attr: str
     :param rule: Rule to check the foreign key against.
@@ -459,8 +567,7 @@ class Relation(Rule):
         self.rule = rule
 
     def __repr__(self):
-        return "Relation({!r}, {!r})".format(
-            self.attr, self.rule)
+        return "Relation({!r}, {!r})".format(self.attr, self.rule)
 
     def query(self, user):
         related_q = self.rule.query(user)
@@ -477,6 +584,18 @@ class Relation(Rule):
 class ManyRelation(Rule):
     """Check that a rule applies to a many-object relationship.
 
+    .. deprecated:: 0.9
+
+        Use :class:`~bridgekeeper.rules.R` objects instead.
+
+        ::
+
+            # old
+            ManyRelation('agency', Is(lambda user: user.agency))
+
+            # new
+            R(agency=lambda user: user.agency)
+
     This can be used in a similar fashion to :class:`Relation`, but
     across a :class:`~django.db.models.ManyToManyField`, or the remote
     end of a :class:`~django.db.models.ForeignKey`.
@@ -507,8 +626,7 @@ class ManyRelation(Rule):
         self.rule = rule
 
     def __repr__(self):
-        return "ManyRelation({!r}, {!r})".format(
-            self.query_attr, self.rule)
+        return "ManyRelation({!r}, {!r})".format(self.query_attr, self.rule)
 
     def query(self, user):
         # Unfortunately you can't use Q objects on a relation, only proper
@@ -524,8 +642,6 @@ class ManyRelation(Rule):
         related_q = self.rule.query(user)
         if related_q is UNIVERSAL or related_q is EMPTY:
             return related_q
-        attr = instance.__class__._meta.get_field(
-            self.query_attr,
-        ).get_accessor_name()
+        attr = instance.__class__._meta.get_field(self.query_attr,).get_accessor_name()
         related_manager = getattr(instance, attr)
         return related_manager.filter(related_q).exists()
